"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Copyright (C) 2018 Silas B. Domingos
 * This source code is licensed under the MIT License as described in the file LICENSE.
 */
const Class = require("@singleware/class");
const DOM = require("@singleware/jsx");
/**
 * Dialog template class.
 */
let Template = class Template {
    /**
     * Default constructor.
     * @param properties Dialog properties.
     * @param children Dialog children.
     */
    constructor(properties, children) {
        /**
         * Header element.
         */
        this.header = (DOM.create("div", { class: "header" },
            DOM.create("slot", { name: "header" })));
        /**
         * Content element.
         */
        this.content = (DOM.create("div", { class: "content" },
            DOM.create("slot", { name: "content" })));
        /**
         * Footer element.
         */
        this.footer = (DOM.create("div", { class: "footer" },
            DOM.create("slot", { name: "footer" })));
        /**
         * Modal element.
         */
        this.modal = (DOM.create("div", { class: "modal" },
            DOM.create("slot", { name: "modal" })));
        /**
         * Dialog element.
         */
        this.dialog = (DOM.create("div", { class: "dialog" },
            this.header,
            this.content,
            this.footer));
        /**
         * Dialog styles.
         */
        this.styles = (DOM.create("style", null, `
:host > .modal {
  position: fixed;
  top: 0;
  left: 0;
  width: 100vw;
  height: 100vh;
  z-index: 999999999;
}
:host > .dialog {
  display: flex;
  position: absolute;
  flex-direction: column;
  overflow: auto;
  top: 45%;
  left: 50%;
  transform: translate(-50%, -50%);
  z-index: 1000000000;
}
:host > .dialog > .header, :host > .dialog > .content, :host > .dialog > .footer {
  display: flex;
  align-items: center;
  flex-direction: column;
}
`));
        this.properties = Object.freeze(properties || {});
        this.skeleton = DOM.create("div", { class: this.properties.class }, children);
        this.elements = this.skeleton.attachShadow({ mode: 'closed' });
        this.elements.appendChild(this.styles);
        this.initialize();
    }
    /**
     * Initializes the custom element.
     */
    initialize() {
        Object.defineProperties(this.skeleton, {
            show: { value: this.show.bind(this), writable: false, configurable: false },
            hide: { value: this.hide.bind(this), writable: false, configurable: false }
        });
    }
    /**
     * Show the dialog.
     * @param modal Determines whether the dialog is shown as modal or not.
     */
    show(modal) {
        if (modal) {
            this.elements.appendChild(this.modal);
        }
        this.elements.appendChild(this.dialog);
    }
    /**
     * Hide the dialog.
     */
    hide() {
        this.modal.remove();
        this.dialog.remove();
    }
    /**
     * Dialog element.
     */
    get element() {
        return this.skeleton;
    }
};
__decorate([
    Class.Public()
], Template.prototype, "properties", void 0);
__decorate([
    Class.Private()
], Template.prototype, "skeleton", void 0);
__decorate([
    Class.Private()
], Template.prototype, "elements", void 0);
__decorate([
    Class.Private()
], Template.prototype, "header", void 0);
__decorate([
    Class.Private()
], Template.prototype, "content", void 0);
__decorate([
    Class.Private()
], Template.prototype, "footer", void 0);
__decorate([
    Class.Private()
], Template.prototype, "modal", void 0);
__decorate([
    Class.Private()
], Template.prototype, "dialog", void 0);
__decorate([
    Class.Private()
], Template.prototype, "styles", void 0);
__decorate([
    Class.Private()
], Template.prototype, "initialize", null);
__decorate([
    Class.Public()
], Template.prototype, "show", null);
__decorate([
    Class.Public()
], Template.prototype, "hide", null);
__decorate([
    Class.Public()
], Template.prototype, "element", null);
Template = __decorate([
    Class.Describe()
], Template);
exports.Template = Template;
